#!/usr/bin/python

from launchpadlib.launchpad import Launchpad
from ConfigParser import ConfigParser
from argparse import ArgumentParser
import optparse
from leankitkanban.kanban import (
    LeankitKanban,
    Record,
)

import datetime

def kanban_parse_config(config):
    """Parse the kanban configuration from `config`.
    """
    globals = {}
    for key, value in config.items('KANBAN'):
        globals[key] = value

    projects = {}
    for section in config.sections():
        if section in ['KANBAN']:
            continue
        projects[section] = {}
        for key in config.defaults():
            projects[section][key] = config.get(section, key)
        for key in config.options(section):
            projects[section][key] = config.get(section, key)
    return globals, projects

def check_card_status(card,  bug):
    print "Checking status of card:"
    print card.title
    print "comparing to bug:"
    print bug.title
    # TODO implement this, move to another lane when done in LP?

def add_card(lane,  bug):
    print u"Adding card for bug: " + unicode(bug.title) + u" in lane " + unicode(lane.title)
    added_card = lane.addCard()
    print "Setting parameters for the card"
    #print bug.title
    #print bug.description
    #print bug.id
    #print bug.owner.display_name
    #print bug.web_link
    added_card.title = bug.title
    added_card.description = u'Launchpad bug: ' + bug.title + u'\n\n' + unicode(bug.web_link)
    added_card.priority = 1
    added_card.size = 1
    added_card.external_system_name = 'Launchpad'
    added_card.external_system_url = str(bug.web_link)
    print added_card.external_system_url
    #added_card._raw_data['ExternalLink'] = bug.web_link
    #added_card._raw_data['AssignedUserName'] = bug.owner.display_name
    #print added_card._raw_data
    print "Saving the changes"
    added_card.save()

def check_and_add(tagged_bugs):
    '''Check if card already exists'''
    
    print "Fetching " + bugboard + "..."
    board = kanban.getBoard(title=bugboard)

    bug_found = False
    # Check all lanes for existing cards
    for laneid, l in board.lanes.iteritems():
        print "Fetching lane by id " + str(laneid)
        if laneid != 0:
            lane = board.getLane(int(laneid))
        else:
            continue
        if str(lane.id) == str(laneid):
            print "Searching lane " + lane.title
            for bug in tagged_bugs:
                ownername = bug.owner.display_name
                title = bug.title
                for card in lane.cards:
                    if card.title == bug.title:
                        bug_found = True
                        print "Existing card found on lane: " + title + " " + ownername
                        check_card_status(card,  bug)

        else:
            print "Lane fetching problem"
        
    if bug_found == False:
        print ""
        print "Card not found"
        print "Bug filing lane: " + str(buglaneid)
        buglane = board.getLane(int(buglaneid))
        add_card(buglane,  bug)
   


def lp_bugtagsync(lp,  kanbancfg, name_pattern=None):
    '''Create Kanban cards based on tagged bugs in LP.'''
    
    global_config, projects_config = kanban_parse_config(kanbancfg)
    project_names = projects_config.keys()
    global bugboard, buglaneid,  kanban, project_name
    kanbanuser = global_config['user']
    kanbanpassword = global_config['password']
    kanban = LeankitKanban(global_config['account'], kanbanuser, kanbanpassword)

    for project_name in project_names:
        bugtag = projects_config[project_name].get('bugtag')
        if bugtag == None:
            print "bugtag parameter not found"
            continue
        lp_projectname = projects_config[project_name].get('buglaunchpadproject')
        bugboard = projects_config[project_name].get('bugboard')
        buglaneid = projects_config[project_name].get('buglaneid')
        
        if bugtag != None and lp_projectname != None:
            lp_project = lp.projects[lp_projectname]

            if lp_project != None:
                print "bugtag " + bugtag + " projectname " + lp_projectname + " lp rojects "
                tagged_bugs = lp_project.searchTasks(tags=bugtag)
                print tagged_bugs
                
                check_and_add(tagged_bugs)
         
def parse_argv():
    '''Parse CLI arguments.

    Return (options, args) tuple.
    '''
    optparser = optparse.OptionParser()
    optparser.add_option('-k', '--kanbanconfig',
        help='Path to Kanban configuration file', dest='kanbanconfig', metavar='PATH')

    (opts, args) = optparser.parse_args()

    if not opts.kanbanconfig:
        optparser.error('No kanban config given')

    return opts, args


def main():

    (opts, args) = parse_argv()

    kanbancfg = ConfigParser()
    kanbancfg.read(opts.kanbanconfig)
    lp = Launchpad.login_anonymously('just testing', 'production')
    lp_bugtagsync(lp,  kanbancfg)    
    
if __name__ == '__main__':
    main()
